#ifndef BWIDGETS_WIDGET_IMPL_HPP
#define BWIDGETS_WIDGET_IMPL_HPP

#include <basic_widgets/w/base/widget.hpp>
#include <basic_widgets/w/feat/event_handler_impl.hpp>

namespace bwidgets
{
    class WidgetImpl : public virtual Widget,
                       public virtual EventHandlerImpl
    {
    public:
        WidgetImpl(Widget* p = nullptr);

        [[nodiscard]] auto parent() -> Widget* override;
        void               parent(Widget*) override;
        void               render() override;
        [[nodiscard]] auto renderer() -> const std::shared_ptr<Renderer>& override;
        void               renderer(std::shared_ptr<Renderer>) override;
        [[nodiscard]] auto theme() const -> const std::shared_ptr<Theme>& override;
        void               theme(std::shared_ptr<Theme>) override;
        void               viewport(const SDL_Rect&) override;
        [[nodiscard]] auto viewport() const -> const SDL_Rect& override;

    protected:
        // Called on viewport change.
        virtual void _handle_geometry_change(const SDL_Rect&) = 0;
        // Called on renderer change.
        virtual void _handle_renderer_change(const std::shared_ptr<Renderer>&) {}
        // Called on rendering.
        virtual void _handle_rendering() = 0;
        virtual void _handle_theme_change(const std::shared_ptr<Theme>&) {}

    private:
        Widget*                   _parent;
        std::shared_ptr<Renderer> _renderer;
        std::shared_ptr<Theme>    _theme;
        SDL_Rect                  _viewport {0, 0, 0, 0};
    };
}

#endif
