#ifndef BWIDGETS_SCROLLBAR_IMPL_HPP
#define BWIDGETS_SCROLLBAR_IMPL_HPP

#include <functional>
#include <memory>

#include <basic_widgets/w/base/widget_impl.hpp>
#include <basic_widgets/w/button.hpp>
#include <basic_widgets/w/feat/mouse_handler_impl.hpp>
#include <basic_widgets/w/scrollbar.hpp>

namespace bwidgets
{
    class ScrollBarImpl : public virtual ScrollBar,
                          public virtual WidgetImpl,
                          public virtual MouseHandlerImpl
    {
    public:
        ScrollBarImpl(Widget* parent = nullptr);

        [[nodiscard]] auto cursor_area() const -> SDL_Rect;
        void               handle_event(const SDL_Event&) override;
        void               on_scroll(std::function<void(float)>) override;
        [[nodiscard]] auto orientation() const -> Orientation override;
        void               orientation(Orientation) override;
        [[nodiscard]] auto size() const noexcept -> Size override;
        [[nodiscard]] auto step() const -> float override;
        void               step(float) override;
        [[nodiscard]] auto value() const -> float override;
        void               value(float) override;

    protected:
        SDL_Rect                   _bar_area;
        std::shared_ptr<Button>    _button_minus;
        std::shared_ptr<Button>    _button_plus;
        const Size                 _cursor_size {16, 16};
        // TODO: cursor_area
        std::function<void(float)> _on_scroll;
        Orientation                _orientation;
        float                      _step;
        float                      _value {0};

        void _handle_geometry_change(const SDL_Rect&) override;
        void _handle_renderer_change(const std::shared_ptr<Renderer>&) override;
        void _handle_rendering() override;
        void _handle_theme_change(const std::shared_ptr<Theme>&) override;
    };
}

#endif
