#include <basic_widgets/w/base/widget_impl.hpp>

using namespace bwidgets;

WidgetImpl::WidgetImpl(Widget* p) : _parent {p}, _theme {nullptr} {}

auto WidgetImpl::parent() -> Widget*
{
    return _parent;
}

void WidgetImpl::parent(Widget* p)
{
    _parent = p;
}

void WidgetImpl::render()
{
    if (!_renderer) return;

    _renderer->viewport(&_viewport);
    _handle_rendering();
}

auto WidgetImpl::renderer() -> const std::shared_ptr<Renderer>&
{
    return _renderer;
}

void WidgetImpl::renderer(std::shared_ptr<Renderer> r)
{
    if (r != _renderer) {
        _handle_renderer_change(r);
        _renderer = std::move(r);
    }
}

auto WidgetImpl::theme() const -> const std::shared_ptr<Theme>&
{
    return _theme;
}

void WidgetImpl::theme(std::shared_ptr<Theme> t)
{
    if (t != _theme) {
        _handle_theme_change(t);
        _theme = t;
    }
}

void WidgetImpl::viewport(const SDL_Rect& vp)
{
    _handle_geometry_change(vp);
    _viewport = vp;
}

auto WidgetImpl::viewport() const -> const SDL_Rect&
{
    return _viewport;
}
