#include <basic_widgets/w/default_theme.hpp>

using namespace bwidgets;
using namespace std;

const Color DefaultTheme::_color_font_bg         = {200, 200, 200, SDL_ALPHA_OPAQUE};
const Color DefaultTheme::_color_font_fg         = {60, 60, 60, SDL_ALPHA_OPAQUE};
const Color DefaultTheme::_color_input_bg        = {170, 170, 170, SDL_ALPHA_OPAQUE};
const Color DefaultTheme::_color_input_bg_active = _color_font_bg;
const Color DefaultTheme::_color_input_fg        = _color_font_fg;
const Color DefaultTheme::_color_widget_bg       = {160, 160, 160, SDL_ALPHA_OPAQUE};
const Color DefaultTheme::_color_widget_bg_dark  = {100, 100, 100, SDL_ALPHA_OPAQUE};
const Color DefaultTheme::_color_widget_bg_hl    = {180, 180, 180, SDL_ALPHA_OPAQUE};
const Color DefaultTheme::_color_widget_border   = {130, 130, 130, SDL_ALPHA_OPAQUE};
const Color DefaultTheme::_color_widget_border_pushed = {90, 90, 90, SDL_ALPHA_OPAQUE};
const Size  DefaultTheme::_size_layout_margin         = {4, 4};
const int   DefaultTheme::_size_widget_border         = 4;

auto DefaultTheme::color_font_bg() const -> const Color&
{
    return _color_font_bg;
}

auto DefaultTheme::color_font_fg() const -> const Color&
{
    return _color_font_fg;
}

auto DefaultTheme::color_input_bg() const -> const Color&
{
    return _color_input_bg;
}

auto DefaultTheme::color_input_bg_active() const -> const Color&
{
    return _color_input_bg_active;
}

auto DefaultTheme::color_input_fg() const -> const Color&
{
    return _color_input_fg;
}

auto DefaultTheme::color_widget_bg_dark() const -> const Color&
{
    return _color_widget_bg_dark;
}

auto DefaultTheme::color_widget_bg() const -> const Color&
{
    return _color_widget_bg;
}

auto DefaultTheme::color_widget_bg_hl() const -> const Color&
{
    return _color_widget_bg_hl;
}

auto DefaultTheme::color_widget_border() const -> const Color&
{
    return _color_widget_border;
}

auto DefaultTheme::color_widget_border_pushed() const -> const Color&
{
    return _color_widget_border_pushed;
}

auto DefaultTheme::font(FontEnum f) -> const shared_ptr<Font>&
{
    if (!_fonts.contains(f) || !_fonts[f]) {
        _fonts.insert_or_assign(f, [f]() {
            switch (f) {
                case FontEnum::MONO:
                    return make_shared<Font>(Font::find("Monospace"), 14);
                case FontEnum::MONO_BIG:
                    return make_shared<Font>(Font::find("Monospace"), 18);
                case FontEnum::MONO_BIGGER:
                    return make_shared<Font>(Font::find("Monospace"), 24);
                case FontEnum::MONO_BIGGEST:
                    return make_shared<Font>(Font::find("Monospace"), 32);
                case FontEnum::MONO_SMALL:
                    return make_shared<Font>(Font::find("Monospace"), 12);
                case FontEnum::MONO_SMALLER:
                    return make_shared<Font>(Font::find("Monospace"), 10);
                case FontEnum::MONO_SMALLEST:
                    return make_shared<Font>(Font::find("Monospace"), 8);
            }
        }());
    }

    return _fonts[f];
}

auto DefaultTheme::font_default() -> const shared_ptr<Font>&
{
    return font(FontEnum::MONO);
}

auto DefaultTheme::font_default_big() -> const shared_ptr<Font>&
{
    return font(FontEnum::MONO_BIG);
}

auto DefaultTheme::font_default_bigger() -> const shared_ptr<Font>&
{
    return font(FontEnum::MONO_BIGGER);
}

auto DefaultTheme::font_default_biggest() -> const shared_ptr<Font>&
{
    return font(FontEnum::MONO_BIGGEST);
}

auto DefaultTheme::font_default_small() -> const shared_ptr<Font>&
{
    return font(FontEnum::MONO_SMALL);
}

auto DefaultTheme::font_default_smaller() -> const shared_ptr<Font>&
{
    return font(FontEnum::MONO_SMALLER);
}

auto DefaultTheme::font_default_smallest() -> const shared_ptr<Font>&
{
    return font(FontEnum::MONO_SMALLEST);
}

auto DefaultTheme::size_layout_margin() const -> const Size&
{
    return _size_layout_margin;
}

auto DefaultTheme::size_widget_border() const -> int
{
    return _size_widget_border;
}

